@@ -7,28 +7,75 @@
 (require 'cl-lib)
 (require 'ert)
 (require 'emacsql)
+
 (require 'emacsql-sqlite)
-(require 'emacsql-psql)
+(if (require 'sqlite nil t)
+    (require 'emacsql-sqlite-builtin)
+  (message "WARNING: Forgo testing `%s' because `%s' is unavailable"
+           'emacsql-sqlite-builtin 'sqlite))
+(if (require 'sqlite3 nil t)
+    (require 'emacsql-sqlite-module)
+  (message "WARNING: Forgo testing `%s' because `%s' is unavailable"
+           'emacsql-sqlite-module 'sqlite3))
 (require 'emacsql-mysql)
-(when (require 'pg nil t) (require 'emacsql-pg))
+(require 'emacsql-psql)
+;; FIXME(CI) Broken and thus disabled in test.yml.
+(cond
+ ((< emacs-major-version 28)
+  (message "WARNING: Forgo testing `%s' because it is unsupported on Emacs %s"
+           'emacsql-pg emacs-version))
+ ((require 'pg nil t)
+  (require 'emacsql-pg))
+ ((message "WARNING: Forgo testing `%s' because `%s' is unavailable"
+           'emacsql-pg 'pg)))
 
 (defvar emacsql-tests-timeout 4
   "Be aggressive about not waiting on subprocesses in unit tests.")
 
 (defvar emacsql-tests-connection-factories
   (let ((factories ())
-        (pgdatabase (getenv "PGDATABASE"))
-        (pguser (getenv "PGUSER"))
-        (mysql-dbname (getenv "MYSQL_DBNAME")))
+        (mysql-database (getenv "MYSQL_DATABASE"))
+        (mysql-user     (getenv "MYSQL_USER"))
+        (mysql-password (getenv "MYSQL_PASSWORD"))
+        (mysql-host     (getenv "MYSQL_HOST"))
+        (mysql-port     (getenv "MYSQL_PORT"))
+        (psql-database  (getenv "PSQL_DATABASE"))
+        (psql-user      (getenv "PSQL_USER"))
+        (psql-host      (getenv "PSQL_HOST"))
+        (psql-port      (getenv "PSQL_PORT"))
+        (pg-database    (getenv "PG_DATABASE"))
+        (pg-user        (getenv "PG_USER"))
+        (pg-password    (getenv "PG_PASSWORD"))
+        (pg-host        (getenv "PG_HOST"))
+        (pg-port        (getenv "PG_PORT")))
     (cl-labels ((reg (name &rest args)
                   (push (cons name (apply #'apply-partially args)) factories)))
       (reg "sqlite" #'emacsql-sqlite nil)
-      (when pgdatabase
-        (reg "psql" #'emacsql-psql pgdatabase))
-      (when (and pgdatabase pguser (fboundp 'emacsql-pg))
-        (reg "pg" #'emacsql-pg pgdatabase pguser))
-      (when mysql-dbname
-        (reg "mysql" #'emacsql-mysql mysql-dbname)))
+      (when (and (featurep 'emacsql-sqlite-builtin)
+                 (fboundp 'sqlite-available-p)
+                 (sqlite-available-p))
+        (reg "sqlite-builtin" 'emacsql-sqlite-builtin nil))
+      (when (and (featurep 'emacsql-sqlite-module)
+                 (boundp 'module-file-suffix)
+                 module-file-suffix)
+        (reg "sqlite-module" 'emacsql-sqlite-module nil))
+      (when (and mysql-database mysql-user mysql-host mysql-password mysql-port)
+        (reg "mysql" #'emacsql-mysql mysql-database
+             :user mysql-user
+             :host mysql-host
+             :password mysql-password
+             :port mysql-port))
+      (when (and psql-database psql-user psql-host psql-port)
+        (reg "psql" #'emacsql-psql psql-database
+             :username psql-user
+             :hostname psql-host
+             :port psql-port))
+      (when (and pg-database pg-user pg-password pg-host pg-port
+                 (fboundp 'emacsql-pg))
+        (reg "pg" #'emacsql-pg pg-database pg-user
+             :host pg-host
+             :password pg-password
+             :port pg-port)))
     (nreverse factories))
   "List of connection factories to use in unit tests.")
 
@@ -45,6 +92,8 @@
 
 (ert-deftest emacsql-nul-character ()
   "Try inserting and retrieving strings with a NUL byte."
+  ;; FIXME #102
+  :expected-result (if (= emacs-major-version 25) :failed :passed)
   (let ((emacsql-global-timeout emacsql-tests-timeout))
     (dolist (factory emacsql-tests-connection-factories)
       (emacsql-with-connection (db (funcall (cdr factory)))
@@ -99,7 +148,8 @@
       (emacsql-with-connection (db (funcall (cdr factory)))
         (emacsql db [:create-temporary-table test-table ([x])])
         (emacsql db [:insert-into test-table :values ([""] [\])])
-        (should (process-live-p (emacsql-process db)))
+        (when (cl-typep db 'process)
+          (should (emacsql-live-p db)))
         (should (equal (emacsql db [:select * :from test-table])
                        '(("") (\))))))))
 
